<?php

namespace Drupal\paid_ads\Controller;

use Drupal\Component\Serialization\Json;
use Drupal\Core\Controller\ControllerBase;
use Drupal\field\Entity\FieldConfig;
use Drupal\paid_ads\Entity\PaidPayment;
use Drupal\paid_ads\PaidService;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Symfony\Component\HttpFoundation\JsonResponse;
use Symfony\Component\HttpFoundation\Request;

/**
 * Handle payment operation routes.
 */
class PaidController extends ControllerBase {

  private $paidService;

  /**
   * PaidController constructor.
   *
   * @param \Drupal\paid_ads\PaidService $paidService
   *   Injection PaidService.
   */
  public function __construct(PaidService $paidService) {
    $this->paidService = $paidService;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    /* @var $paidService \Drupal\paid_ads\PaidService */
    $paidService = $container->get('paid.service');

    return new static($paidService);
  }

  /**
   * Handle creating a payment route.
   *
   * @param \Symfony\Component\HttpFoundation\Request $request
   *   Request from client.
   *
   * @return \Symfony\Component\HttpFoundation\Response
   *   Response as JSON.
   */
  public function createPayment(Request $request) {
    $options = Json::decode($request->get('options'));
    try {
      $plugin = $this->paidService->getInstance($request->get('type'));
      $field_id = $options['data_entity_type'] . '.' . $options['data_bundle'] . '.' . $options['data_field'];
      $fieldConfig = FieldConfig::load($field_id);
      $amount = $this->paidService->getAmountByFieldClass($fieldConfig->getSetting('class'), ['field_id' => $field_id]);
      if ($plugin && $amount) {

        $payment = PaidPayment::create()
          ->setPaymentMethod($plugin)
          ->setAmount($amount)
          ->setOptions([
            'onSuccess' => [
              [$fieldConfig->getSetting('class'), 'onSuccess'],
              $options,
            ],
          ]);
        $payment->save();
        return $plugin->onCreatePayment([
          'request' => $request,
          'payment' => $payment,
        ]);
      }
    }
    catch (\Exception $exception) {
      // TODO: handle the exception.
    }

    return $this->sendErrorResponse();
  }

  /**
   * Handle execution payment route.
   *
   * @param \Symfony\Component\HttpFoundation\Request $request
   *   Request from client.
   *
   * @return \Symfony\Component\HttpFoundation\Response
   *   Response as JSON.
   */
  public function executePayment(Request $request) {
    try {
      $plugin = $this->paidService->getInstance($request->get('type'));
      if ($plugin) {
        return $plugin->onExecutePayment(['request' => $request]);
      }
    }
    catch (\Exception $exception) {
    }

    return $this->sendErrorResponse();
  }

  /**
   * Return response with 500 error code.
   *
   * @return \Symfony\Component\HttpFoundation\JsonResponse
   *   Response as JSON with 500 error.
   */
  public function sendErrorResponse() {
    return JsonResponse::create(['status' => 'error'], 500);
  }

}
