<?php

namespace Drupal\paid_ads\Plugin\PaidGateways;

use Drupal\Component\Serialization\Json;
use Drupal\Core\Annotation\Translation;
use Drupal\Core\Form\FormStateInterface;
use Drupal\paid_ads\Annotation\PaidGateway;
use Drupal\paid_ads\Entity\PaidPayment;
use Drupal\paid_ads\Plugin\PaidGatewaysPluginBase;
use Symfony\Component\HttpFoundation\JsonResponse;

/**
 * Class PaypalGateways. Implements PayPal REST API.
 *
 * @PaidGateway(
 *   id = "paypal_gateway",
 *   label = @Translation("PayPal")
 * )
 */
class PaypalGateways extends PaidGatewaysPluginBase {

  /**
   * {@inheritdoc}
   */
  public function getForm(array $option) {
    $amount = $option['amount'] ?? '0.01';
    return [
      '#theme' => 'paypal_paid',
      '#API_key' => $this->getClientId(),
      '#amount' => $amount . '$',
      '#attached' => [
        'library' => [
          'paid_ads/paid_paypal',
        ],
      ],

    ];
  }

  /**
   * {@inheritdoc}
   */
  public function buildConfigurationForm(array $form, FormStateInterface $form_state) {
    $values = $this->getOptions();
    $form['mode'] = [
      '#title' => $this->t('Mode'),
      '#type' => 'select',
      '#options' => [
        'https://api.sandbox.paypal.com' => $this->t('Sandbox'),
        'https://api.paypal.com' => $this->t('Live'),
      ],
    ];

    $form['client_id'] = [
      '#title' => $this->t('Client ID'),
      '#type' => 'textfield',
      '#default_value' => $values['client_id'] ?? '',
    ];

    $form['secret_id'] = [
      '#title' => $this->t('Secret ID'),
      '#type' => 'textfield',
      '#default_value' => $values['secret_id'] ?? '',
    ];

    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function submitConfigurationForm(array &$form, FormStateInterface $form_state) {
    $values = $form_state->getValue('form');
    $this->setOptions($values);
  }

  /**
   * {@inheritdoc}
   */
  public function getPluginId() {
    return $this->pluginId;
  }

  /**
   * {@inheritdoc}
   */
  public function getPluginDefinition() {
    return [];
  }

  /**
   * Gets selected endpoint.
   *
   * @return string
   *   API endpoint.
   */
  private function getEndpoint() {
    return $this->getOptions()['mode'] ?? 'https://api.sandbox.paypal.com';
  }

  /**
   * Gets client ID.
   *
   * @return string
   *   Selected client ID.
   */
  private function getClientId() {
    return $this->getOptions()['client_id'] ?? '';
  }

  /**
   * Gets secrete ID.
   *
   * @return string
   *   Secret ID.
   */
  private function getSecretId() {
    return $this->getOptions()['secret_id'] ?? '';
  }

  /**
   * {@inheritdoc}
   */
  public function onCreatePayment(array $context) {
    /* @var PaidPayment $payment */
    $payment = $context['payment'];
    $amount = $payment->getAmount();
    $client = \Drupal::httpClient();
    $request = \Drupal::request();

    $jsonArray = [
      'intent' => 'sale',
      'payer' => [
        'payment_method' => 'paypal',
      ],
      'transactions' => [
        [
          'amount' => [
            'total' => $amount,
            'currency' => 'USD',
          ],
          'invoice_number' => $payment->getId(),
        ],
      ],
      'redirect_urls' => [
        "return_url" => $request->getHost(),
        "cancel_url" => $request->getHost(),
      ],
    ];
    try {
      $postResponse = $client->post($this->getEndpoint() . '/v1/payments/payment',
        [
          'auth' => [$this->getClientId(), $this->getSecretId()],
          'json' => $jsonArray,
        ]);
      $response = Json::decode($postResponse->getBody());

    }
    catch (\Exception $exception) {
      // TODO: exception handler.
      return JsonResponse::create([], 500);
    }

    return JsonResponse::create($response);
  }

  /**
   * {@inheritdoc}
   */
  public function onExecutePayment(array $context) {
    $client = \Drupal::httpClient();
    $request = $context['request'];
    $paymentId = $request->get('paymentID');
    $payerId = $request->get('payerID');

    $jsonArray = [
      'payer_id' => $payerId,
    ];

    $response = $client->post(
      $this->getEndpoint() . '/v1/payments/payment/' . $paymentId . '/execute',
      [
        'auth' => [$this->getClientId(), $this->getSecretId()],
        'json' => $jsonArray,
        'headers' => [
          'Content-Type' => 'application/json',
        ],
      ]);
    $params = Json::decode($response->getBody());
    if ($params['state'] == 'approved') {
      foreach ($params['transactions'] as $transaction) {
        $payment = PaidPayment::load($transaction['invoice_number']);
        $payment->execute();
      }
    }
    return JsonResponse::create(['status' => 'ok']);
  }

}
